@@ -1,6 +1,6 @@
 /*
  * File:	feature.h
- * $Id: feature.h,v 1.16 1998/11/17 19:44:10 mason Exp $
+ * $Id: feature.h,v 1.20.2.4 1999/08/17 07:02:45 mason Exp $
  *
  * Compile-time configuration.
  *-----------------------------------------------------------------------
@@ -161,7 +161,7 @@
  * instead of
  *	Home = "\E[7~", End = "\E[8~"	[default]
  */
-#define LINUX_KEYS
+/* #define LINUX_KEYS */
 
 /*
  * Enable the keysym resource which allows you to define strings associated
@@ -172,11 +172,25 @@
 # define KEYSYM_RESOURCE
 #endif
 
+/*
+ * Modifier/s to use to allow up/down arrows and Priot/Next keys
+ * to scroll single or page-fulls
+ */
+#define SCROLL_ON_SHIFT
+/* #define SCROLL_ON_CTRL */
+/* #define SCROLL_ON_META */
+
 /*
  * Allow unshifted Next/Prior keys to scroll forward/back
  * (in addition to shift+Next/shift+Prior)       --pjh
  */
 /* #define UNSHIFTED_SCROLLKEYS */
+
+/* (Hops) Set to choose a number of lines of context between pages 
+ *      (rather than a proportion (1/5) of savedlines buffer) 
+ *      when paging the savedlines with SHIFT-{Prior,Next} keys.
+ */
+#define PAGING_CONTEXT_LINES 1 /* */
   
 /*--------------------------------MOUSE---------------------------------*/
 /*
@@ -232,7 +246,8 @@
 /*-----------------------------SCROLL BAR-------------------------------*/
 /*
  * Choose the scrollbar width - should be an even number [default: 10]
- * Except for XTERM_SCROLLBAR: it is *always* 15.
+ * Except for XTERM_SCROLLBAR: it is *always* 15
+ * 	and for NEXT_SCROLLBAR, which is *always* 19
  */
 /* #define SB_WIDTH 10 */
 
@@ -289,6 +304,11 @@
  */
 #define INEXPENSIVE_LOCAL_X_CALLS
 
+/*
+ * Force local connection to be socket (or other local) communication
+ */
+#define LOCAL_X_IS_UNIX
+
 /*
  * Have DISPLAY environment variable & "\E[7n" transmit display with IP number
  */
@@ -327,11 +347,12 @@
  */
 #define SAVELINES	64
 
-/* (Hops) Set to choose a number of lines of context between pages 
- *      (rather than a proportion (1/5) of savedlines buffer) 
- *      when paging the savedlines with SHIFT-{Prior,Next} keys.
+/*
+ * Provide support for pathetic applications which expect specifically
+ * undefined "bw" (termcap/terminfo) behaviour to be specifically defined
+ * as xterm defines it.
  */
-#define PAGING_CONTEXT_LINES 1 /* */
+#define SUPPORT_BROKEN_APPS_WHICH_RELY_ON_UNDEFINED_BW_BEHAVIOUR_AS_XTERM
 
 /*
  * List of default fonts available
@@ -340,24 +361,30 @@
  * Sizes between multi-char fonts sets (MFONT_LIST) and single-char font
  * sets (NFONT_LIST) have been matched up
  */
+#ifndef MULTICHAR_SET		/* no Kanji or Big5 or GB support */
+# define NFONTS		5
+# define FONT0_IDX	2
+# undef  MFONT_LIST
+# define NFONT_LIST	"7x14", "6x10", "6x13", "8x13", "9x15"
+#endif
 #ifdef KANJI
 # define NFONTS		5
 # define FONT0_IDX	2
 # define MFONT_LIST	"k14", "jiskan16", "jiskan18", "jiskan24", "jiskan26"
 # define NFONT_LIST	"7x14", "8x16", "9x18", "12x24", "13x26"
-#else
-# ifdef ZH
-#  define NFONTS	5
-#  define FONT0_IDX	1
-#  define MFONT_LIST	"taipei16", "taipeik20", "taipeik24", "taipeik20", \
-			"taipei16"
-#  define NFONT_LIST	"8x16", "10x20", "12x24", "10x20", "8x16"
-# else				/* no Kanji or Big5 support */
-#  define NFONTS	5
-#  define FONT0_IDX	2
-#  undef  MFONT_LIST
-#  define NFONT_LIST	"7x14", "6x10", "6x13", "8x13", "9x15"
-# endif
+#endif
+#ifdef ZH
+# define NFONTS		5
+# define FONT0_IDX	1
+# define MFONT_LIST	"taipei16", "taipeik20", "taipeik24", "taipeik20", \
+       			"taipei16"
+# define NFONT_LIST	"8x16", "10x20", "12x24", "10x20", "8x16"
+#endif
+#ifdef ZHCN		/* Here are our default GB fonts. */
+# define NFONTS		3
+# define FONT0_IDX	1
+# define MFONT_LIST	"hanzigb16st", "hanzigb24st", "hanzigb16fs"
+# define NFONT_LIST	"8x16", "12x24", "8x16"
 #endif
 
 #endif
