@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	main.c
  *----------------------------------------------------------------------*
- * $Id: main.c,v 1.52 1998/11/17 19:44:10 mason Exp $
+ * $Id: main.c,v 1.66.2.7 1999/08/17 08:11:03 mason Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (C) 1992      John Bovey, University of Canterbury
@@ -125,15 +125,14 @@ color_aliases(int idx)
 /*
  * find if fg/bg matches any of the normal (low-intensity) colors
  */
-#ifndef NO_BRIGHTCOLOR
 /* INTPROTO */
 void
 set_colorfgbg(void)
 {
     unsigned int    i;
-    const char      colorfgbg_env[] = "COLORFGBG=default;default;bg";
     char           *p;
     int             fg = -1, bg = -1;
+    static char     env_colorfgbg[] = "COLORFGBG=default;default;bg";
 
     for (i = Color_Black; i <= Color_White; i++) {
 	if (PixColors[Color_fg] == PixColors[i]) {
@@ -148,7 +147,7 @@ set_colorfgbg(void)
 	}
     }
 
-    p = strchr(colorfgbg_env, '=');
+    p = strchr(env_colorfgbg, '=');
     p++;
     if (fg >= 0)
 	sprintf(p, "%d;", fg);
@@ -157,14 +156,15 @@ set_colorfgbg(void)
     p = strchr(p, '\0');
     if (bg >= 0)
 	sprintf(p,
-# ifdef XPM_BACKGROUND
+#ifdef XPM_BACKGROUND
 		"default;"
-# endif
+#endif
 		"%d", bg);
     else
 	STRCPY(p, "default");
-    putenv(colorfgbg_env);
+    putenv(env_colorfgbg);
 
+#ifndef NO_BRIGHTCOLOR
     colorfgbg = DEFAULT_RSTYLE;
     for (i = minCOLOR; i <= maxCOLOR; i++) {
 	if (PixColors[Color_fg] == PixColors[i]
@@ -180,10 +180,8 @@ set_colorfgbg(void)
 	if (PixColors[Color_bg] == PixColors[i])
 	    colorfgbg = SET_BGCOLOR(colorfgbg, i);
     }
+#endif
 }
-#else				/* NO_BRIGHTCOLOR */
-# define set_colorfgbg() ((void)0)
-#endif				/* NO_BRIGHTCOLOR */
 
 /* INTPROTO */
 void
@@ -198,8 +196,8 @@ Get_Colours(void)
 	if (!rs[Rs_color + i])
 	    continue;
 
-	if (!XParseColor(Xdisplay, Xcmap, rs[Rs_color + i], &xcol) ||
-	    !XAllocColor(Xdisplay, Xcmap, &xcol)) {
+	if (!XParseColor(Xdisplay, Xcmap, rs[Rs_color + i], &xcol)
+	    || !XAllocColor(Xdisplay, Xcmap, &xcol)) {
 	    print_error(msg, rs[Rs_color + i]);
 #ifndef XTERM_REVERSE_VIDEO
 	    if (i < 2 && (Options & Opt_reverseVideo)) {
@@ -209,8 +207,8 @@ Get_Colours(void)
 	        rs[Rs_color + i] = def_colorName[i];
 	    if (!rs[Rs_color + i])
 		continue;
-	    if (!XParseColor(Xdisplay, Xcmap, rs[Rs_color + i], &xcol) ||
-		!XAllocColor(Xdisplay, Xcmap, &xcol)) {
+	    if (!XParseColor(Xdisplay, Xcmap, rs[Rs_color + i], &xcol)
+		|| !XAllocColor(Xdisplay, Xcmap, &xcol)) {
 		print_error(msg, rs[Rs_color + i]);
 		switch (i) {
 		case Color_fg:
@@ -345,7 +343,7 @@ Create_Windows(int argc, const char * const *argv)
     attributes.background_pixel = PixColors[Color_fg];
     attributes.border_pixel = PixColors[Color_fg];
     attributes.colormap = Xcmap;
-    TermWin.parent = XCreateWindow(Xdisplay, Xroot,
+    TermWin.parent[0] = XCreateWindow(Xdisplay, Xroot,
 				   szHint.x, szHint.y,
 				   szHint.width, szHint.height,
 				   BORDERWIDTH,
@@ -354,7 +352,7 @@ Create_Windows(int argc, const char * const *argv)
 				   CWBackPixel | CWBorderPixel | CWColormap,
 				   &attributes);
 #else
-    TermWin.parent = XCreateSimpleWindow(Xdisplay, Xroot,
+    TermWin.parent[0] = XCreateSimpleWindow(Xdisplay, Xroot,
 					 szHint.x, szHint.y,
 					 szHint.width, szHint.height,
 					 BORDERWIDTH,
@@ -368,16 +366,16 @@ Create_Windows(int argc, const char * const *argv)
     classHint.res_class = (char *) APL_CLASS;
     wmHint.input = True;
     wmHint.initial_state = (Options & Opt_iconic ? IconicState : NormalState);
-    wmHint.window_group = TermWin.parent;
+    wmHint.window_group = TermWin.parent[0];
     wmHint.flags = (InputHint | StateHint | WindowGroupHint);
 
-    XSetWMProperties(Xdisplay, TermWin.parent, NULL, NULL, (char **) argv,
+    XSetWMProperties(Xdisplay, TermWin.parent[0], NULL, NULL, (char **) argv,
 		     argc, &szHint, &wmHint, &classHint);
 
-    XSelectInput(Xdisplay, TermWin.parent,
-		 (KeyPressMask | FocusChangeMask |
-		  StructureNotifyMask | VisibilityChangeMask)
-	);
+    XSelectInput(Xdisplay, TermWin.parent[0],
+		 (KeyPressMask | FocusChangeMask
+		  | VisibilityChangeMask
+		  | StructureNotifyMask));
 
 /* vt cursor: Black-on-White is standard, but this is more popular */
     TermWin_cursor = XCreateFontCursor(Xdisplay, XC_xterm);
@@ -395,7 +393,7 @@ Create_Windows(int argc, const char * const *argv)
     cursor = XCreateFontCursor(Xdisplay, XC_left_ptr);
 
 /* the vt window */
-    TermWin.vt = XCreateSimpleWindow(Xdisplay, TermWin.parent,
+    TermWin.vt = XCreateSimpleWindow(Xdisplay, TermWin.parent[0],
 				     0, 0,
 				     szHint.width, szHint.height,
 				     0,
@@ -408,7 +406,7 @@ Create_Windows(int argc, const char * const *argv)
 		  Button1MotionMask | Button3MotionMask));
 
 /* scrollBar: size doesn't matter */
-    scrollBar.win = XCreateSimpleWindow(Xdisplay, TermWin.parent,
+    scrollBar.win = XCreateSimpleWindow(Xdisplay, TermWin.parent[0],
 					0, 0,
 					1, 1,
 					0,
@@ -425,7 +423,7 @@ Create_Windows(int argc, const char * const *argv)
     }
 
 #ifdef XPM_BACKGROUND
-    if (rs[Rs_backgroundPixmap] != NULL) {
+    if (rs[Rs_backgroundPixmap] != NULL && !(Options & Opt_transparent)) {
 	const char     *p = rs[Rs_backgroundPixmap];
 
 	if ((p = strchr(p, ';')) != NULL) {
@@ -478,7 +476,7 @@ resize_subwindows(int width, int height)
 #ifndef XTERM_SCROLLBAR
     /* arrows are as high as wide - leave 1 pixel gap */
 # ifdef NEXT_SCROLLBAR
-	scrollBar.end -= ((SB_WIDTH * 2) + 2) + sb_shadow;
+	scrollBar.end -= ((SB_WIDTH - 1) * 2);
 # else
 	scrollBar.beg += (SB_WIDTH + 1) + sb_shadow;
 	scrollBar.end -= (SB_WIDTH + 1) + sb_shadow;
@@ -505,9 +503,10 @@ resize_subwindows(int width, int height)
 
     if (old_width)
 	Gr_Resize(old_width, old_height);
-    XClearWindow(Xdisplay, TermWin.vt);
+
+    scr_clear();
     resize_pixmap();
-    XSync(Xdisplay, 0);
+    XSync(Xdisplay, False);
 }
 
 /* EXTPROTO */
@@ -515,19 +514,19 @@ void
 resize_all_windows(void)
 {
     szhints_recalc();
-    XSetWMNormalHints(Xdisplay, TermWin.parent, &szHint);
+    XSetWMNormalHints(Xdisplay, TermWin.parent[0], &szHint);
     AddToCNQueue(szHint.width, szHint.height);
-    XResizeWindow(Xdisplay, TermWin.parent, szHint.width, szHint.height);
-    resize_subwindows(szHint.width, szHint.height);
+    XResizeWindow(Xdisplay, TermWin.parent[0], szHint.width, szHint.height);
+    resize_window(szHint.width, szHint.height);
 }
 
 /*
  * Redraw window after exposure or size change
  * width/height are those of the parent
  */
-/* INTPROTO */
+/* EXTPROTO */
 void
-resize_window1(unsigned int width, unsigned int height)
+resize_window(unsigned int width, unsigned int height)
 {
     int             new_ncol, new_nrow;
     static int      old_width, old_height = -1;
@@ -558,24 +557,6 @@ resize_window1(unsigned int width, unsigned int height)
     old_height = height;
 }
 
-/*
- * Redraw window after exposure or size change
- */
-/* EXTPROTO */
-void
-resize_window(void)
-{
-    Window          root;
-    int             unused_x, unused_y;
-    unsigned int    unused_border, unused_depth, width, height;
-
-    XGetGeometry(Xdisplay, TermWin.parent,
-		 &root, &unused_x, &unused_y, &width, &height,
-		 &unused_border, &unused_depth);
-/* parent already resized */
-    resize_window1(width, height);
-}
-
 /*
  * Set the width/height of the window in characters.  Units are pixels.
  * good for toggling 80/132 columns
@@ -599,8 +580,11 @@ set_widthheight(unsigned int width, unsigned int height)
 	height = szHint.base_height + height;
 
 	AddToCNQueue(width, height);
-	XResizeWindow(Xdisplay, TermWin.parent, width, height);
-	resize_window1(width, height);
+	XResizeWindow(Xdisplay, TermWin.parent[0], width, height);
+	resize_window(width, height);
+#ifdef USE_XIM
+	IMSetStatusPosition();
+#endif
     }
 }
 
@@ -673,14 +657,14 @@ void
 set_title(const char *str)
 {
 #ifndef SMART_WINDOW_TITLE
-    XStoreName(Xdisplay, TermWin.parent, str);
+    XStoreName(Xdisplay, TermWin.parent[0], str);
 #else
     char           *name;
 
-    if (XFetchName(Xdisplay, TermWin.parent, &name))
+    if (XFetchName(Xdisplay, TermWin.parent[0], &name))
 	name = NULL;
     if (name == NULL || strcmp(name, str))
-	XStoreName(Xdisplay, TermWin.parent, str);
+	XStoreName(Xdisplay, TermWin.parent[0], str);
     if (name)
 	XFree(name);
 #endif
@@ -691,14 +675,14 @@ void
 set_iconName(const char *str)
 {
 #ifndef SMART_WINDOW_TITLE
-    XSetIconName(Xdisplay, TermWin.parent, str);
+    XSetIconName(Xdisplay, TermWin.parent[0], str);
 #else
     char           *name;
 
-    if (XGetIconName(Xdisplay, TermWin.parent, &name))
+    if (XGetIconName(Xdisplay, TermWin.parent[0], &name))
 	name = NULL;
     if (name == NULL || strcmp(name, str))
-	XSetIconName(Xdisplay, TermWin.parent, str);
+	XSetIconName(Xdisplay, TermWin.parent[0], str);
     if (name)
 	XFree(name);
 #endif
@@ -731,8 +715,8 @@ set_window_color(int idx, const char *color)
 	    goto Done;
 	}
     }
-    if (!XParseColor(Xdisplay, Xcmap, color, &xcol) ||
-	!XAllocColor(Xdisplay, Xcmap, &xcol)) {
+    if (!XParseColor(Xdisplay, Xcmap, color, &xcol)
+	|| !XAllocColor(Xdisplay, Xcmap, &xcol)) {
 	print_error(msg, color);
 	return;
     }
@@ -758,7 +742,7 @@ set_window_color(int idx, const char *color)
 /* XSetWindowAttributes attr; */
 /* Cursor cursor; */
   Done:
-    if (idx == Color_bg)
+    if (idx == Color_bg && !(Options & Opt_transparent))
 	XSetWindowBackground(Xdisplay, TermWin.vt, PixColors[Color_bg]);
 
 /* handle Color_BD, scrollbar background, etc. */
@@ -980,7 +964,11 @@ change_font(int init, const char *fontname)
     xfont = XLoadQueryFont(Xdisplay, rs[Rs_mfont + idx]);
     if (!xfont) {
 	print_error(msg, rs[Rs_mfont + idx]);
+#ifdef ZHCN
+	rs[Rs_mfont + idx] = "-*-16-*-gb2312*-*";
+#else
 	rs[Rs_mfont + idx] = "k14";
+#endif
 	xfont = XLoadQueryFont(Xdisplay, rs[Rs_mfont + idx]);
 	if (!xfont) {
 	    print_error(msg, rs[Rs_mfont + idx]);
@@ -1008,8 +996,9 @@ change_font(int init, const char *fontname)
 	    TermWin.fprop = 1;	/* Proportional font */
 
 	if (TermWin.fprop == 1)	/* also, if == 0, per_char[] might be NULL */
-	    for (i = TermWin.font->min_char_or_byte2;
-		 i <= TermWin.font->max_char_or_byte2; i++) {
+	    for (i = TermWin.font->max_char_or_byte2
+		     - TermWin.font->min_char_or_byte2;
+		 --i >= 0; ) {
 		cw = TermWin.font->per_char[i].width;
 		MAX_IT(fw, cw);
 	    }
@@ -1033,8 +1022,9 @@ change_font(int init, const char *fontname)
 	    if (fw != boldFont->max_bounds.width)
 		fw = -1;
 	} else
-	    for (i = TermWin.font->min_char_or_byte2;
-		 i <= TermWin.font->max_char_or_byte2; i++) {
+	    for (i = TermWin.font->max_char_or_byte2
+		     - TermWin.font->min_char_or_byte2;
+		 --i >= 0; ) {
 		cw = boldFont->per_char[i].width;
 		MAX_IT(fw, cw);
 	    }
@@ -1050,8 +1040,10 @@ change_font(int init, const char *fontname)
     TermWin.width = TermWin.ncol * TermWin.fwidth;
     TermWin.height = TermWin.nrow * TermWin.fheight;
 
-    if (!init)
+    if (!init) {
 	resize_all_windows();
+	scr_touch();
+    }
     return;
   Abort:
     print_error("aborting");	/* fatal problem */
@@ -1066,10 +1058,12 @@ change_font(int init, const char *fontname)
 void
 init_vars(void)
 {
-    Options = Opt_scrollBar;
+    Options = Opt_scrollBar | Opt_scrollTtyOutput;
     sb_shadow = 0;
     TermWin.ncol = 80;
     TermWin.nrow = 24;
+    TermWin.mapped = 0;
+    want_refresh = 1;
     scrollBar.win = 0;
 #if (MENUBAR_MAX)
     menuBar.win = 0;
@@ -1131,9 +1125,15 @@ init_resources(int argc, const char * const *argv)
  */
     if ((rs[Rs_display_name] = getenv("DISPLAY")) == NULL)
 	rs[Rs_display_name] = ":0";
+#ifdef LOCAL_X_IS_UNIX
+    if (strncmp(rs[Rs_display_name], ":0", 2) == 0)
+	rs[Rs_display_name] = "unix:0";
+#endif
 
     get_options(r_argc, r_argv);
 
+    FREE(r_argv);
+
     if ((Xdisplay = XOpenDisplay(rs[Rs_display_name])) == NULL) {
 	print_error("can't open display %s", rs[Rs_display_name]);
 	exit(EXIT_FAILURE);
@@ -1145,8 +1145,9 @@ init_resources(int argc, const char * const *argv)
 
     extract_resources(Xdisplay, rs[Rs_name]);
 
-#ifndef XTERM_SCROLLBAR
-    sb_shadow = (Options & Opt_scrollBar_floating) ? 0 : SHADOW;
+#if ! defined(XTERM_SCROLLBAR) && ! defined(NEXT_SCROLLBAR)
+    if (!(Options & Opt_scrollBar_floating))
+	sb_shadow = SHADOW;
 #endif
     
 /*
@@ -1219,6 +1220,7 @@ init_resources(int argc, const char * const *argv)
 	    rs[Rs_mfont + i] = def_mfontName[i];
 #endif
     }
+    TermWin.fontset = NULL;
 
 #ifdef XTERM_REVERSE_VIDEO
 /* this is how xterm implements reverseVideo */
@@ -1264,8 +1266,9 @@ init_env(void)
 {
     int             i;
     unsigned int    u;
-    char           *windowid_string, *display_string, *term_string;
     char           *val;
+/* these don't need to be static but do so to placate some mem checkers */
+    static char    *env_windowid, *env_display, *env_term;
 
 #ifdef DISPLAY_IS_IP
 /* Fixup display_name for export over pty to any interested terminal
@@ -1286,13 +1289,15 @@ init_env(void)
 	rs[Rs_display_name] = val;	/* use broken `:0' value */
 
     i = strlen(val);
-    display_string = MALLOC((i + 9) * sizeof(char));
-    sprintf(display_string, "DISPLAY=%s", val);
+    env_display = MALLOC((i + 9) * sizeof(char));
+    sprintf(env_display, "DISPLAY=%s", val);
 
-    /* avoiding the math library: i = (int)(ceil(log10((unsigned int)-1))) */
-    for (i = 0, u = -1; u; u /= 10, i++);
-    windowid_string = MALLOC((i + 10) * sizeof(char));
-    sprintf(windowid_string, "WINDOWID=%u", (unsigned int)TermWin.parent);
+    /* avoiding the math library:
+     * i = (int)(ceil(log10((unsigned int)TermWin.parent[0]))) */
+    for (i = 0, u = (unsigned int)TermWin.parent[0]; u; u /= 10, i++);
+    MAX_IT(i, 1);
+    env_windowid = MALLOC((i + 10) * sizeof(char));
+    sprintf(env_windowid, "WINDOWID=%u", (unsigned int)TermWin.parent[0]);
 
 /* add entries to the environment:
  * @ DISPLAY:   in case we started with -display
@@ -1301,8 +1306,8 @@ init_env(void)
  * @ TERM:      terminal name
  * @ TERMINFO:	path to terminfo directory
  */
-    putenv(display_string);
-    putenv(windowid_string);
+    putenv(env_display);
+    putenv(env_windowid);
 #ifdef RXVT_TERMINFO
     putenv("TERMINFO=" RXVT_TERMINFO);
 #endif
@@ -1311,9 +1316,9 @@ init_env(void)
     else
 	putenv("COLORTERM=" COLORTERMENVFULL);
     if (rs[Rs_term_name] != NULL) {
-	term_string = MALLOC((strlen(rs[Rs_term_name]) + 6) * sizeof(char));
-	sprintf(term_string, "TERM=%s", rs[Rs_term_name]);
-	putenv(term_string);
+	env_term = MALLOC((strlen(rs[Rs_term_name]) + 6) * sizeof(char));
+	sprintf(env_term, "TERM=%s", rs[Rs_term_name]);
+	putenv(env_term);
     } else {
 	putenv("TERM=" TERMENV);
     }
@@ -1347,6 +1352,8 @@ main(int argc, const char * const *argv)
 
     Create_Windows(argc, argv);
 
+    init_xlocale();
+
     scr_reset();		/* initialize screen */
     Gr_reset();			/* reset graphics */
 
@@ -1362,9 +1369,16 @@ main(int argc, const char * const *argv)
 #if (MENUBAR_MAX)
     if (menubar_visible())
 	XMapWindow(Xdisplay, menuBar.win);
+#endif
+#ifdef TRANSPARENT
+    if (Options & Opt_transparent) {
+	XSetWindowBackgroundPixmap(Xdisplay, TermWin.parent[0], ParentRelative);
+	XSetWindowBackgroundPixmap(Xdisplay, TermWin.vt, ParentRelative);
+	XSelectInput(Xdisplay, Xroot, PropertyChangeMask);
+    }
 #endif
     XMapWindow(Xdisplay, TermWin.vt);
-    XMapWindow(Xdisplay, TermWin.parent);
+    XMapWindow(Xdisplay, TermWin.parent[0]);
 
     init_env();
     init_command(cmd_argv);
