@@ -1,11 +1,35 @@
 /*
- * $Id: rxvt.h,v 1.29 1998/11/17 19:44:10 mason Exp $
+ * $Id: rxvt.h,v 1.40.2.6 1999/07/17 09:43:31 mason Exp $
  */
 
 #ifndef _RXVT_H			/* include once only */
 #define _RXVT_H
 
 #include "../config.h"
+
+/* sort out conflicts in feature.h */
+#undef  MULTICHAR_SET		/* a glyph is only ever defined by 1 char */
+#ifdef KANJI
+# define MULTICHAR_SET		/* a glyph is defined by 1 or 2 chars     */
+# undef ZH			/* remove Chinese big5 support            */
+# undef ZHCN			/* remove Chinese gb support	          */
+# undef GREEK_SUPPORT		/* Kanji/Greek together is too weird      */
+# undef DEFINE_XTERM_COLOR	/* since kterm-color doesn't exist?       */
+#endif
+#ifdef ZH 
+# define MULTICHAR_SET		/* a glyph is defined by 1 or 2 chars     */
+# undef KANJI			/* can't put Chinese/Kanji together       */
+# undef ZHCN
+# undef GREEK_SUPPORT		
+# undef DEFINE_XTERM_COLOR
+#endif
+#ifdef ZHCN
+# define MULTICHAR_SET		/* a glyph is defined by 1 or 2 chars     */
+# undef KANJI
+# undef ZH
+# undef GREEK_SUPPORT
+#endif
+
 #include "feature.h"
 
 /*
@@ -39,11 +63,6 @@
 #include <sys/strredir.h>
 #endif
 
-#ifdef _SCO_DS     /* SCO Osr5 - Hops */
-    /* Scos pty naming is /dev/[pt]typ0 - /dev/[pt]ty256 */
-# define ALL_NUMERIC_PTYS
-#endif 
-
 /*
  *****************************************************************************
  * INCLUDES
@@ -104,28 +123,17 @@
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
 
+#include "rsizes.h"		/* we've pulled in <sys/types.h> - presumed */
+#ifndef HAVE_XPOINTER
+typedef char           *XPointer;
+#endif
+
 #ifdef RXVT_GRAPHICS
 # include "rxvtgrx.h"
 #endif
 #ifdef GREEK_SUPPORT
 # include "grkelot.h"
 #endif
-#if defined(KANJI) || defined(ZH)
-# define MULTICHAR_SET		/* a glyph is defined by 1 or 2 chars */
-#else
-# undef  MULTICHAR_SET		/* a glyph is only ever defined by 1 char */
-#endif
-
-/*
- * Play some size tricks:  <X11/Xmd.h> might be better
- * Now if only we could presume every OS has int16_t, u_int16_t, etc.
- */
-typedef RINT16T		R_int16_t;
-typedef RUINT16T	R_u_int16_t;
-typedef RINT32T		R_int32_t;
-typedef RUINT32T	R_u_int32_t;
-typedef RINTPT		R_int_p_t;
-typedef RUINTPT		R_u_int_p_t;
 
 /*
  *****************************************************************************
@@ -154,18 +162,26 @@ typedef struct {
 # undef KEEP_SCROLLCOLOR
 #endif
 
+#ifdef TRANSPARENT
+# define KNOW_PARENTS		4
+#else
+# define KNOW_PARENTS		1
+#endif
+
 typedef struct {
-    R_int16_t       width,	/* window width [pixels]                    */
+    int16_t         width,	/* window width [pixels]                    */
 		    height,	/* window height [pixels]                   */
 		    fwidth,	/* font width [pixels]                      */
 		    fheight,	/* font height [pixels]                     */
 		    fprop,	/* font is proportional                     */
 		    ncol, nrow,	/* window size [characters]                 */
 		    focus,	/* window has focus                         */
-		    saveLines,	/* number of lines that fit in scrollback   */
-		    nscrolled,	/* number of line actually scrolled         */
+		    mapped,	/* window state mapped?                     */
+		    saveLines;	/* number of lines that fit in scrollback   */
+    u_int16_t       nscrolled,	/* number of line actually scrolled         */
 		    view_start;	/* scrollback view starts here              */
-    Window          parent, vt;	/* parent (main) and vt100 window           */
+    Window          parent[KNOW_PARENTS], /* parent[0] is our window        */
+		    vt;		/* vt100 window                             */
     GC              gc;		/* GC for drawing text                      */
     XFontStruct    *font;	/* main font structure                      */
 #ifndef NO_BOLDFONT
@@ -174,6 +190,7 @@ typedef struct {
 #ifdef MULTICHAR_SET
     XFontStruct    *mfont;	/* Multichar font structure                 */
 #endif
+    XFontSet       fontset;
 #ifdef XPM_BACKGROUND
     Pixmap          pixmap;
 #ifdef XPM_BUFFERING
@@ -209,7 +226,7 @@ typedef struct grwin_t {
 } grwin_t;
 
 typedef struct {
-    R_int16_t       row, col;
+    int16_t         row, col;
 } row_col_t;
 
 
@@ -235,18 +252,6 @@ typedef struct {
 # error if you disable both selection styles, how can you select, silly?
 #endif
 
-/* sort out conflicts in feature.h */
-#ifdef KANJI
-# undef ZH			/* remove Chinese big5 support        */
-# undef GREEK_SUPPORT		/* Kanji/Greek together is too weird  */
-# undef DEFINE_XTERM_COLOR	/* since kterm-color doesn't exist?   */
-#endif
-
-#ifdef ZH 
-# undef KANJI			/* can't put Chinese/Kanji together   */
-# undef GREEK_SUPPORT		
-# undef DEFINE_XTERM_COLOR
-#endif
 
 #ifndef XPM_BACKGROUND
 # undef XPM_BUFFERING		 /* disable what can't be used */
@@ -304,15 +309,20 @@ typedef struct {
 # define SHADOW 2
 #endif
 
-#ifdef XTERM_SCROLLBAR
-# undef  SB_WIDTH
-# define SB_WIDTH		15
+#ifdef NEXT_SCROLLBAR
+# undef SB_WIDTH
+# define SB_WIDTH		19
 #else
-# if !defined (SB_WIDTH) || (SB_WIDTH < 8)
-#  undef SB_WIDTH
-#  define SB_WIDTH		11	/* scrollBar width */
-# endif
-#endif				/* XTERM_SCROLLBAR */
+# ifdef XTERM_SCROLLBAR
+#  undef  SB_WIDTH
+#  define SB_WIDTH		15
+# else
+#  if !defined (SB_WIDTH) || (SB_WIDTH < 8)
+#   undef SB_WIDTH
+#   define SB_WIDTH		11	/* scrollBar width */
+#  endif
+# endif				/* XTERM_SCROLLBAR */
+#endif
 
 #define NO_REFRESH		0	/* Window not visible at all!        */
 #define FAST_REFRESH		(1<<1)  /* Fully exposed window              */
@@ -395,9 +405,11 @@ enum {
 #define Opt_scrollBar_right	(1LU<<8)
 #define Opt_scrollBar_floating	(1LU<<9)
 #define Opt_meta8		(1LU<<10)
-#define Opt_scrollTtyOutputInh	(1LU<<11)
+#define Opt_scrollTtyOutput	(1LU<<11)
 #define Opt_scrollKeypress	(1LU<<12)
+#define Opt_transparent		(1LU<<13)
 /* place holder used for parsing command-line options */
+#define Opt_Reverse		(1LU<<30)
 #define Opt_Boolean		(1LU<<31)
 
 /*
@@ -512,7 +524,7 @@ enum Rs_resource_list {
     Rs_scrollBar,
     Rs_scrollBar_right,
     Rs_scrollBar_floating,
-    Rs_scrollTtyOutputInh,
+    Rs_scrollTtyOutput,
     Rs_scrollKeypress,
     Rs_saveLines,
     Rs_utmpInhibit,
@@ -533,13 +545,19 @@ enum Rs_resource_list {
 #ifdef PRINTPIPE
     Rs_print_pipe,
 #endif
+#ifdef USE_XIM
+    Rs_preeditType,
+    Rs_inputMethod,
+#endif
 #if defined (HOTKEY_CTRL) || defined (HOTKEY_META)
     Rs_bigfont_key,
     Rs_smallfont_key,
 #endif
-#ifdef CUTCHAR_RESOURCE
-    Rs_cutchars,
+#ifdef TRANSPARENT
+    Rs_transparent,
 #endif
+    Rs_cutchars,
+    Rs_modifier,
     TOTAL_RS
 } ;
 
@@ -560,7 +578,7 @@ enum Rs_resource_list {
 #ifdef NO_RMEMSET
 # define MEMSET(a, b, c)	memset ((a), (b), (c))
 #else
-# define MEMSET(a, b, c)	rmemset ((a), (b), (R_int_p_t) (c))
+# define MEMSET(a, b, c)	rmemset ((a), (b), (intp_t) (c))
 #endif
 
 #define MALLOC(sz)		malloc (sz)
@@ -588,7 +606,10 @@ enum Rs_resource_list {
 #define GET_FGCOLOR(r)		(((r) & RS_fgMask))
 #define GET_BGCOLOR(r)		(((r) & RS_bgMask)>>5)
 #define GET_ATTR(r)		(((r) & RS_attrMask))
-#define GET_BGATTR(r)		(((r) & (RS_attrMask | RS_bgMask)))
+#define GET_BGATTR(r)							\
+    (((r) & RS_RVid) ? (((r) & (RS_attrMask & ~RS_RVid))		\
+			| (((r) & RS_fgMask)<<5))			\
+		     : ((r) & (RS_attrMask | RS_bgMask)))
 #define SET_FGCOLOR(r,fg)	(((r) & ~RS_fgMask)  | (fg))
 #define SET_BGCOLOR(r,bg)	(((r) & ~RS_bgMask)  | ((bg)<<5))
 #define SET_ATTR(r,a)		(((r) & ~RS_attrMask)| (a))
@@ -610,15 +631,17 @@ enum Rs_resource_list {
 # define scrollbar_upButton(y)	((y) > scrollBar.end \
 				 && (y) <= scrollbar_dnval())
 # define scrollbar_dnButton(y)	((y) > scrollbar_dnval())
+# define SCROLL_MINHEIGHT	15
 #else
 # define scrollbar_upButton(y)	((y) < scrollBar.beg)
 # define scrollbar_dnButton(y)	((y) > scrollBar.end)
+# define SCROLL_MINHEIGHT	10
 #endif
 
 #define scrollbar_above_slider(y)	((y) < scrollBar.top)
 #define scrollbar_below_slider(y)	((y) > scrollBar.bot)
 #define scrollbar_position(y)		((y) - scrollBar.beg)
-#define scrollbar_size()		(scrollBar.end - scrollBar.beg)
+#define scrollbar_size()		(scrollBar.end - scrollBar.beg - SCROLL_MINHEIGHT)
 
 #if (MENUBAR_MAX > 1)
 /* rendition style flags */
@@ -677,6 +700,7 @@ EXTERN Pixel		PixColors[TOTAL_COLORS];
 #ifdef INEXPENSIVE_LOCAL_X_CALLS
 EXTERN int              display_is_local;
 #endif
+EXTERN short		want_refresh;
 
 EXTERN const char      *rs[TOTAL_RS];
 
