@@ -1,7 +1,7 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	xdefaults.c
  *----------------------------------------------------------------------*
- * $Id: xdefaults.c,v 1.29 1998/11/16 07:07:39 mason Exp $
+ * $Id: xdefaults.c,v 1.34.2.3 1999/07/28 20:22:39 mason Exp $
  *
  * All portions of code are copyright by their respective author/s.
  * Copyright (C) 1994      Robert Nation <nation@rocket.sanders.lockheed.com>
@@ -71,6 +71,8 @@
     (optList[i].flag ? 0 : (optList[i].arg ? strlen (optList[i].arg) : 1))
 #define optList_isBool(i)						\
     (optList[i].flag & Opt_Boolean)
+#define optList_isReverse(i)						\
+    (optList[i].flag & Opt_Reverse)
 #define optList_size()							\
     (sizeof(optList) / sizeof(optList[0]))
 
@@ -100,10 +102,17 @@ static const struct {
          "scrollbar right"),
     BOOL(Rs_scrollBar_floating, "scrollBar_floating", "st",
          Opt_scrollBar_floating, "scrollbar without a trough"),
-    BOOL(Rs_scrollTtyOutputInh, "scrollTtyOutput", "si",
-         Opt_scrollTtyOutputInh, "scroll-on-tty-output inhibit"),
+    BOOL(Rs_scrollTtyOutput, "scrollTtyOutput", NULL,
+         Opt_scrollTtyOutput, NULL),
+    SWCH("si", Opt_Boolean|Opt_Reverse|Opt_scrollTtyOutput,
+	 "scroll-on-tty-output inhibit"),
     BOOL(Rs_scrollKeypress, "scrollTtyKeypress", "sk",
          Opt_scrollKeypress, "scroll-on-keypress"),
+#ifdef TRANSPARENT
+    BOOL(Rs_transparent, "inheritPixmap", "ip",
+	 Opt_transparent, "inherit parent pixmap"),
+    SWCH("tr", Opt_transparent, NULL),
+#endif
     BOOL(Rs_utmpInhibit, "utmpInhibit", "ut", Opt_utmpInhibit,
          "utmp inhibit"),
     BOOL(Rs_visualBell, "visualBell", "vb", Opt_visualBell, "visual bell"),
@@ -203,11 +212,16 @@ static const struct {
     RSTRG(Rs_mfont + 7, "mfont7", "fontname"),
 # endif
 #endif				/* MULTICHAR_SET */
-
 #ifdef MULTICHAR_SET
     STRG(Rs_multichar_encoding, "multichar_encoding", "km", "mode",
-         "multiple-character font encoding; mode = eucj | sjis | big5"),
+         "multiple-character font encoding; mode = eucj | sjis | big5 | gb"),
 #endif				/* MULTICHAR_SET */
+#ifdef USE_XIM
+    STRG(Rs_preeditType, "preeditType", "pt", "style",
+         "input style of input method; style = OverTheSpot | OffTheSpot | Root"),
+    STRG(Rs_inputMethod, "inputMethod", "im", "name",
+         "name of input method"),
+#endif                         /* USE_XIM */
 #ifdef GREEK_SUPPORT
     STRG(Rs_greek_keyboard, "greek_keyboard", "grk", "mode",
          "greek keyboard mapping; mode = iso | ibm"),
@@ -243,6 +257,7 @@ static const struct {
     RSTRG(Rs_bigfont_key, "bigfont_key", "keysym"),
     RSTRG(Rs_smallfont_key, "smallfont_key", "keysym"),
 #endif
+    STRG(Rs_modifier, "modifier", "mod", "modifier", "alt | meta | hyper | super | mod1 | ... | mod5"),
 #ifdef CUTCHAR_RESOURCE
     RSTRG(Rs_cutchars, "cutchars", "string"),
 #endif				/* CUTCHAR_RESOURCE */
@@ -285,6 +300,9 @@ usage(int type)
 #ifdef ZH
     fprintf(stderr, "Chinese,");
 #endif
+#ifdef ZHCN
+    fprintf(stderr, "Chinese(GB),");
+#endif
 #ifdef XTERM_SCROLLBAR
     fprintf(stderr, "XTerm-scrollbar,");
 #endif
@@ -423,12 +441,14 @@ get_options(int argc, const char * const *argv)
 
     /* feature: always try to match long-options */
 	for (entry = 0; entry < optList_size(); entry++)
-	    if ((optList[entry].kw && !strcmp(opt, optList[entry].kw)) ||
-		(!longopt &&
-		 optList[entry].opt && !strcmp(opt, optList[entry].opt)))
+	    if ((optList[entry].kw && !strcmp(opt, optList[entry].kw))
+		|| (!longopt
+		    && optList[entry].opt && !strcmp(opt, optList[entry].opt)))
 		break;
 
 	if (entry < optList_size()) {
+	    if (optList_isReverse(entry))
+		flag = flag == On ? Off : On;
 	    if (optList_strlen(entry)) {	/* string value */
 		const char     *str = argv[++i];
 
@@ -649,8 +669,8 @@ get_xdefaults(FILE *stream, const char *name)
 	while (*str && isspace(*str))
 	    str++;		/* leading whitespace */
 
-	if ((str[len] != '*' && str[len] != '.') ||
-	    (len && strncmp(str, name, len)))
+	if ((str[len] != '*' && str[len] != '.')
+	    || (len && strncmp(str, name, len)))
 	    continue;
 	str += (len + 1);	/* skip `name*' or `name.' */
 
@@ -670,12 +690,19 @@ get_xdefaults(FILE *stream, const char *name)
 		    n = strlen(str);
 		    if (n && *(optList[entry].dp) == NULL) {
 		    /* not already set */
+			int             s;
 			char           *p = MALLOC((n + 1) * sizeof(char));
 
 			STRCPY(p, str);
 			*(optList[entry].dp) = p;
 			if (optList_isBool(entry)) {
-			    if (!my_strcasecmp(str, "TRUE"))
+			    s = my_strcasecmp(p, "TRUE") == 0
+				|| my_strcasecmp(p, "YES") == 0
+				|| my_strcasecmp(p, "ON") == 0
+				|| my_strcasecmp(p, "1") == 0;
+			    if (optList_isReverse(entry))
+				s = !s;
+			    if (s)
 				Options |= (optList[entry].flag);
 			    else
 				Options &= ~(optList[entry].flag);
@@ -731,18 +758,25 @@ extract_resources(Display *display, const char *name)
 #  endif
 
     for (entry = 0; entry < optList_size(); entry++) {
+	int             s;
 	char           *p;
 	const char     *kw = optList[entry].kw;
 
 	if (kw == NULL || *(optList[entry].dp) != NULL)
 	    continue;		/* previously set */
-	if ((p = XGetDefault(display, name, kw)) != NULL ||
-	    (p = XGetDefault(display, APL_SUBCLASS, kw)) != NULL ||
-	    (p = XGetDefault(display, APL_CLASS, kw)) != NULL) {
+	if ((p = XGetDefault(display, name, kw)) != NULL
+	    || (p = XGetDefault(display, APL_SUBCLASS, kw)) != NULL
+	    || (p = XGetDefault(display, APL_CLASS, kw)) != NULL) {
 	    *optList[entry].dp = p;
 
 	    if (optList_isBool(entry)) {
-		if (!my_strcasecmp(p, "TRUE"))
+		s = my_strcasecmp(p, "TRUE") == 0
+		    || my_strcasecmp(p, "YES") == 0
+		    || my_strcasecmp(p, "ON") == 0
+		    || my_strcasecmp(p, "1") == 0;
+		if (optList_isReverse(entry))
+		    s = !s;
+		if (s)
 		    Options |= (optList[entry].flag);
 		else
 		    Options &= ~(optList[entry].flag);
@@ -861,14 +895,18 @@ extract_resources(Display *display, const char *name)
     }
 #endif				/* GREEK_SUPPORT */
 
-#define to_keysym(pks,str) do { KeySym sym;\
-if (str && ((sym = XStringToKeysym(str)) != 0)) *pks = sym; } while (0)
-
 #if defined (HOTKEY_CTRL) || defined (HOTKEY_META)
-    to_keysym(&ks_bigfont, rs[Rs_bigfont_key]);
-    to_keysym(&ks_smallfont, rs[Rs_smallfont_key]);
+    {
+	KeySym          sym;
+
+	if (rs[Rs_bigfont_key]
+	    && ((sym = XStringToKeysym(rs[Rs_bigfont_key])) != 0))
+	    ks_bigfont = sym;
+	if (rs[Rs_smallfont_key]
+	    && ((sym = XStringToKeysym(rs[Rs_smallfont_key])) != 0))
+	    ks_smallfont = sym;
+    }
 #endif
-#undef to_keysym
 }
 /*}}} */
 /*----------------------- end-of-file (C source) -----------------------*/
